# -*- coding: utf-8 -*-
import logging.config
import optparse

from pyramid_oereb.standard.xtf_import import FederalTopic


# logging.basicConfig(level=logging.INFO)
logging.config.dictConfig({
    'version': 1,
    'disable_existing_loggers': False,  # this fixes the problem
    'formatters': {
        'standard': {
            'format': '%(asctime)s [%(levelname)s] %(name)s: %(message)s'
        },
    },
    'handlers': {
        'default': {
            'level': 'INFO',
            'class': 'logging.StreamHandler',
            'formatter': 'standard'
        },
    },
    'loggers': {
        'import_federal_topic': {
            'handlers': ['default'],
            'level': 'INFO',
            'propagate': True
        }
    }
})


def run():

    parser = optparse.OptionParser(
        usage='usage: %prog [options] [file1.xml file2.xtf ...]',
        description='Import tool for federal OEREB topics. If no files are specified, the data will be '
                    'downloaded from the configured URL.'
    )
    parser.add_option(
        '-c', '--config',
        dest='config',
        metavar='YAML',
        type='string',
        help='The path to the configuration yaml file.'
    )
    parser.add_option(
        '-s', '--section',
        dest='section',
        metavar='SECTION',
        type='string',
        default='pyramid_oereb',
        help='The section which contains configuration (default is: pyramid_oereb).'
    )
    parser.add_option(
        '-t', '--topic',
        dest='topic_code',
        metavar='TOPIC_CODE',
        type='string',
        help='The topic code; has to be available in configuration!'
    )
    parser.add_option(
        '-f', '--force',
        dest='force',
        action='store_true',
        default=False,
        help='Skip comparing the checksums and force import.'
    )
    parser.add_option(
        '-S', '--srid',
        dest='srid',
        metavar='SRID',
        type='int',
        help='Overwrite SRID set in configuration file.'
    )
    parser.add_option(
        '-a', '--arc-max-diff',
        dest='arc_max_diff',
        metavar='MAX_DIFF',
        type='float',
        default=0.001,
        help='The maximum difference for stroked arcs (default is: 0.001).'
    )
    parser.add_option(
        '-A', '--arc-precision',
        dest='arc_precision',
        metavar='PRECISION',
        type='int',
        default=3,
        help='The decimal precision of generated arc coordinates (default is: 3).'
    )
    parser.add_option(
        '-d', '--temp-dir',
        dest='tmp_dir',
        metavar='TEMP_DIR',
        type='string',
        default='.',
        help='The temporary working directory. (default is ".")'
    )
    parser.add_option(
        '--c2ctemplate-style',
        dest='c2ctemplate_style',
        action='store_true',
        default=False,
        help='Is the yaml file using a c2ctemplate style (starting with vars)'
    )

    options, args = parser.parse_args()
    if not options.config:
        parser.error('No configuration file set.')
    if not options.topic_code:
        parser.error('No topic code defined.')

    download = not (len(args) > 0)

    loader = FederalTopic(
        options.config,
        options.topic_code,
        section=options.section,
        c2ctemplate_style=options.c2ctemplate_style,
        arc_max_diff=options.arc_max_diff,
        arc_precision=options.arc_precision,
        tmp_dir=options.tmp_dir,
        srid=options.srid
    )

    if download:
        loader.download_data()
        loader.unzip_data()
        loader.read_checksum()
        files = loader.collect_files()
    else:
        files = args

    loader.load(files, force=options.force)

    if download:
        loader.cleanup_files()



# -*- coding: utf-8 -*-
import datetime
import logging

from mako import exceptions
from mako.template import Template
from pyramid.httpexceptions import HTTPNotFound
from pyramid.path import AssetResolver, DottedNameResolver
from pyramid.response import Response
from sqlalchemy import cast, Text

from pyramid_oereb import Config, database_adapter, route_prefix
from pyramid_oereb.lib import b64
from pyramid_oereb.lib.records.image import ImageRecord
from pyramid_oereb.lib.records.office import OfficeRecord


log = logging.getLogger(__name__)


def get_logo(request):
    """
    Returns the requested logo.

    Args:
        request (pyramid.request.Request): The request containing the logo key matchdict parameter.

    Returns:
        pyramid.response.Response: The generated response object.
    """
    logo_key = request.matchdict.get('logo')
    logo_language = request.matchdict.get('language')
    if logo_key in Config.get('logo').keys():
        logo = Config.get_logo_config(language=logo_language).get(logo_key)
        response = request.response
        response.status_int = 200
        response.body = logo.content
        response.content_type = logo.mimetype
        return response
    raise HTTPNotFound('This logo does not exist.')


def get_municipality(request):
    """
    Returns the requested municipality logo from database.

    Args:
        request (pyramid.request.Request): The request containing the fosnr as matchdict parameter.

    Returns:
        pyramid.response.Response: The generated response object.
    """
    fosnr = request.matchdict.get('fosnr')
    source_params = Config.get_municipality_config().get('source').get('params')
    session = database_adapter.get_session(source_params.get('db_connection'))
    try:
        model = DottedNameResolver().resolve(source_params.get('model'))
        municipality = session.query(model).filter_by(fosnr=fosnr).first()
        if municipality:
            logo = getattr(municipality, 'logo', None)
            if logo:
                response = request.response
                response.status_int = 200
                response.body = b64.decode(logo)
                response.content_type = ImageRecord.get_mimetype(bytearray(response.body))
                return response
        raise HTTPNotFound()
    finally:
        session.close()


def get_symbol(request):
    """
    Returns the symbol for the requested theme and type code from database.

    Args:
        request (pyramid.request.Request): The request containing the codes as matchdict parameters.

    Returns:
        pyramid.response.Response: The generated response object.
    """

    theme_code = request.matchdict.get('theme_code')
    view_service_id = request.matchdict.get('view_service_id')
    type_code = request.matchdict.get('type_code')

    plr = None
    for p in Config.get('plrs'):
        if str(p.get('code')).lower() == str(theme_code).lower():
            plr = p
            break

    if plr is None:
        raise HTTPNotFound('No theme with code {}.'.format(theme_code))

    source_params = plr.get('source').get('params')
    session = database_adapter.get_session(source_params.get('db_connection'))

    try:
        model = DottedNameResolver().resolve('{module_}.{class_}'.format(
            module_=source_params.get('models'),
            class_='LegendEntry'
        ))
        legend_entry = session.query(model).filter(
            cast(model.type_code, Text) == cast(type_code, Text)
        ).filter(
            model.view_service_id == view_service_id
        ).first()
        if legend_entry:
            symbol = getattr(legend_entry, 'symbol', None)
            if symbol:
                response = request.response
                response.status_int = 200
                response.body = b64.decode(symbol)
                response.content_type = ImageRecord.get_mimetype(bytearray(response.body))
                return response
        raise HTTPNotFound()

    finally:
        session.close()


def get_symbol_ref(request, record):
    """
    Returns the link to the symbol of the specified public law restriction.

    Args:
        request (pyramid.request.Request): The current request instance.
        record (pyramid_oereb.lib.records.plr.PlrRecord or
            pyramid_oereb.lib.records.view_service.LegendEntryRecord): The record of the public law
            restriction to get the symbol reference for.

    Returns:
        uri: The link to the symbol for the specified public law restriction.
    """
    return request.route_url(
        '{0}/image/symbol'.format(route_prefix),
        theme_code=record.theme.code,
        view_service_id=record.view_service_id,
        type_code=record.type_code,
        extension=record.symbol.extension
    )


def get_surveying_data_provider(real_estate):
    """

    Args:
        real_estate (pyramid_oereb.lib.records.real_estate.RealEstateRecord): The real estate for which the
            provider of the surveying data should be delivered.
    Returns:
        provider (pyramid_oereb.lib.records.office.OfficeRecord): The provider who produced the used
            surveying data.
    """
    provider = OfficeRecord({u'de': u'This is only a dummy'})
    return provider


def get_surveying_data_update_date(real_estate):
    """
    Gets the date of the latest update of the used survey data data for the
    situation map. The method you find here is only matching the standard configuration. But you can provide
    your own one if your configuration is different. The only thing you need to take into account is that the
    input of this method is always and only a real estate record. And the output of this method must be a
    datetime.date object.

    Args:
        real_estate (pyramid_oereb.lib.records.real_estate.RealEstateRecord): The real
            estate for which the last update date of the base data should be indicated

    Returns:
        update_date (datetime.datetime): The date of the last update of the cadastral base data
    """

    update_date = datetime.datetime.now()

    return update_date


def produce_sld_content(request):
    """
    This is the standard hook method to provide the sld content. Of course you can set it to another one. For
    instance to use another template. Or to use other parameters to provide the correctly constructed SLD.

    .. note:: What to know about this Method: REQUEST-Method: GET, parameters only as url parameters

    When you are replacing this method take care that it has to accept a pyramid request as input and must
    deliver a valid SLD wrapped in a pyramid response instance as output. It is in your responsibility.

    Args:
        request (pyramid.request.Request): The request from the pyramid application.

    Returns:
        pyramid.response.Response: The
    """
    response = request.response
    template = Template(
        filename=AssetResolver('pyramid_oereb').resolve('standard/templates/sld.xml').abspath(),
        input_encoding='utf-8',
        output_encoding='utf-8'
    )
    layer = Config.get_real_estate_config().get('visualisation').get('layer')
    template_params = {}
    template_params.update(Config.get_real_estate_config().get('visualisation').get('style'))
    template_params.update({'layer_name': layer.get('name')})
    template_params.update({'identifier': request.params.get('egrid')})
    try:
        if isinstance(response, Response) and response.content_type == response.default_content_type:
            response.content_type = 'application/xml'
        response.body = template.render(**template_params)
        return response
    except Exception:
        response.content_type = 'text/html'
        response.body = exceptions.html_error_template().render()
        return response


def plr_sort_within_themes(extract):
    """
    This is the standard hook method to sort a plr list (while respecting the theme order).
    This standard hook does no sorting, you can set your configuration to a different method if you need a
    specific sorting.

    Args:
        extract (pyramid_oereb.lib.records.extract.ExtractRecord): The unsorted extract

    Returns:
        pyramid_oereb.lib.records.extract.ExtractRecord: Returns the updated extract
    """
    return extract



# -*- coding: utf-8 -*-
import codecs
import re
import json
import optparse
import os
import importlib

from sqlalchemy import create_engine
from sqlalchemy.engine import Connection
from sqlalchemy.orm import sessionmaker, class_mapper

from pyramid_oereb.lib.config import Config


class SampleData(object):
    """
    Class for sample data handling.
    """

    def __init__(self, configuration, section='pyramid_oereb', c2ctemplate_style=False,
                 directory='sample_data', sql_file=None):
        """

        Args:
            configuration (str): Path to the configuration yaml file.
            section (str): The used section within the yaml file. Default is `pyramid_oereb`.
            c2ctemplate_style (bool): True if the yaml use a c2c template style (vars.[section]).
                Default is False.
            directory (str): Location of the sample data. Default is `sample_data`.
            sql_file (file): The SQL file to be created. Default is None.
        """
        self._configuration = configuration
        self._section = section
        self._directory = directory
        self._sql_file = sql_file

        Config.init(self._configuration, self._section, c2ctemplate_style)
        self._engine = create_engine(Config.get('app_schema').get('db_connection'), echo=True)
        self._connection = None

    @classmethod
    def _format_value(cls, value):
        """
        Format the value in SQL.

        Args:
            value (str): The value
        Returns:
            str: The formatted value
        """
        if isinstance(value, str) or isinstance(value, bytes):
            return u"'{}'".format(value.replace("'", "''"))
        if isinstance(value, int) or isinstance(value, float):
            return str(value)
        if isinstance(value, bool):
            return "'t'" if value else "'f'"
        if isinstance(value, dict):
            return cls._format_value(json.dumps(value, ensure_ascii=False))
        raise "Unsupported type {}".format(type(value))

    def _do_sql_insert(self, sql, items):
        """
        Write the SQL inserts in a file.

        Args:
            sql (str): The based SQL statement
            items (array): The values
        """
        for k, v in items.items():
            sql = re.sub(r":{}\b".format(re.escape(k)), self._format_value(v), sql)

        for column in re.findall(r":[a-z_0-9]+\b", sql):
            sql = re.sub(r", :?{}\b".format(re.escape(column[1:])), '', sql)
            sql = re.sub(r":?{}, ".format(re.escape(column[1:])), '', sql)

        self._sql_file.write(u"{};\n".format(sql))

    def _has_connection(self):
        """
        Returns True if an active connection is available.

        Returns:
             bool: True if an active connection is available.
        """
        return isinstance(self._connection, Connection) and not self._connection.closed

    def _load_sample(self, class_, import_file_name):
        """
        Load the samples in the database ir in an SQL file.

        Args:
            class_ (sqlalchemy.ext.declarative.ConcreteBase): SQLAlchemy class
            import_file_name (str): The resource file name to import
        """
        with codecs.open(os.path.join(self._directory, import_file_name), encoding='utf-8') as f:
            if self._sql_file is None:
                if self._has_connection():
                    self._connection.execute(class_.__table__.insert(), json.loads(f.read()))
            else:
                sql = str(class_.__table__.insert())
                for r in json.loads(f.read()):
                    self._do_sql_insert(sql, r)

    def _truncate_existing(self, schema):
        """
        Truncates existing tables.
        """

        if self._has_connection():

            # Truncate tables
            self._connection.execute('TRUNCATE {schema}.{table} CASCADE;'.format(
                schema=schema.Glossary.__table__.schema,
                table=schema.Glossary.__table__.name
            ))
            self._connection.execute('TRUNCATE {schema}.{table} CASCADE;'.format(
                schema=schema.ExclusionOfLiability.__table__.schema,
                table=schema.ExclusionOfLiability.__table__.name
            ))
            self._connection.execute('TRUNCATE {schema}.{table} CASCADE;'.format(
                schema=schema.Municipality.__table__.schema,
                table=schema.Municipality.__table__.name
            ))
            self._connection.execute('TRUNCATE {schema}.{table} CASCADE;'.format(
                schema=schema.Address.__table__.schema,
                table=schema.Address.__table__.name
            ))
            self._connection.execute('TRUNCATE {schema}.{table} CASCADE;'.format(
                schema=schema.RealEstate.__table__.schema,
                table=schema.RealEstate.__table__.name
            ))
            self._connection.execute('TRUNCATE {schema}.{table} CASCADE;'.format(
                schema=schema.PublicLawRestrictionDocument.__table__.schema,
                table=schema.PublicLawRestrictionDocument.__table__.name
            ))
            self._connection.execute('TRUNCATE {schema}.{table} CASCADE;'.format(
                schema=schema.DocumentReference.__table__.schema,
                table=schema.DocumentReference.__table__.name
            ))
            self._connection.execute('TRUNCATE {schema}.{table} CASCADE;'.format(
                schema=schema.DocumentBase.__table__.schema,
                table=schema.DocumentBase.__table__.name
            ))
            self._connection.execute('TRUNCATE {schema}.{table} CASCADE;'.format(
                schema=schema.Geometry.__table__.schema,
                table=schema.Geometry.__table__.name
            ))
            self._connection.execute('TRUNCATE {schema}.{table} CASCADE;'.format(
                schema=schema.PublicLawRestriction.__table__.schema,
                table=schema.PublicLawRestriction.__table__.name
            ))
            self._connection.execute('TRUNCATE {schema}.{table} CASCADE;'.format(
                schema=schema.ViewService.__table__.schema,
                table=schema.ViewService.__table__.name
            ))
            self._connection.execute('TRUNCATE {schema}.{table} CASCADE;'.format(
                schema=schema.Office.__table__.schema,
                table=schema.Office.__table__.name
            ))
            self._connection.execute('TRUNCATE {schema}.{table} CASCADE;'.format(
                schema=schema.Availability.__table__.schema,
                table=schema.Availability.__table__.name
            ))
            self._connection.execute('TRUNCATE {schema}.{table} CASCADE;'.format(
                schema=schema.LegendEntry.__table__.schema,
                table=schema.LegendEntry.__table__.name
            ))
            self._connection.execute('TRUNCATE {schema}.{table} CASCADE;'.format(
                schema=schema.DataIntegration.__table__.schema,
                table=schema.DataIntegration.__table__.name
            ))

    def load(self):
        """
        Performs the database operations to load the sample data.
        """
        # Find data model for each PLR from config
        plrs = Config._config.get('plrs')
        models = {plr['code']: plr['source']['params']['models'] for plr in plrs}
        try:
            contaminated_public_transport_sites = \
                importlib.import_module(models['ContaminatedPublicTransportSites'])
            groundwater_protection_zones = importlib.import_module(models['GroundwaterProtectionZones'])
            motorways_building_lines = importlib.import_module(models['MotorwaysBuildingLines'])
            contaminated_military_sites = importlib.import_module(models['ContaminatedMilitarySites'])
            forest_perimeters = importlib.import_module(models['ForestPerimeters'])
        except KeyError as e:
            raise Exception(f"Missing model in YAML configuration file: {e}")

        from pyramid_oereb.standard.models.main import RealEstate, Address, Municipality, \
            Glossary, ExclusionOfLiability

        if self._sql_file is None:
            self._connection = self._engine.connect()

        try:

            for schema, folder in [
                (contaminated_public_transport_sites, "contaminated_public_transport_sites"),
                (groundwater_protection_zones, "groundwater_protection_zones"),
                (forest_perimeters, "forest_perimeters"),
                (motorways_building_lines, "motorways_building_lines"),
                (contaminated_military_sites, "contaminated_military_sites"),
            ]:
                print("Import theme {}.".format(folder))
                # Truncate existing tables
                self._truncate_existing(schema)

                # Fill tables with sample data
                for class_, file_name in [
                    (schema.Availability, 'availabilities.json'),
                    (schema.Office, 'office.json'),
                    (schema.DataIntegration, 'data_integration.json'),
                    (schema.ViewService, 'view_service.json'),
                    (schema.LegendEntry, 'legend_entry.json'),
                    (schema.PublicLawRestriction, 'public_law_restriction.json'),
                    (schema.Geometry, 'geometry.json'),
                ]:
                    self._load_sample(class_, os.path.join('plr119', folder, file_name))

                with open(os.path.join(self._directory, 'plr119', folder, 'document.json')) as f:
                    lps = json.loads(f.read())
                    if self._sql_file is None:
                        Session = sessionmaker(bind=self._engine)  # Use session because of table inheritance
                        session = Session()
                        for lp in lps:
                            session.add(schema.Document(**lp))
                        session.commit()
                        session.close()
                    else:
                        for lp in lps:
                            for table_name in ['document_base', 'document']:
                                table = [
                                    t for t in class_mapper(schema.Document).tables if t.name == table_name
                                ][0]
                                data = {
                                    'type': 'document'
                                }
                                data.update(lp)
                                self._do_sql_insert(str(table.insert()), data)

                if hasattr(schema, 'PublicLawRestrictionDocument'):
                    for class_, file_name in [
                        (schema.PublicLawRestrictionDocument, 'public_law_restriction_document.json'),
                        (schema.DocumentReference, 'document_reference.json'),
                    ]:
                        self._load_sample(class_, os.path.join('plr119', folder, file_name))
            for class_, file_name in [
                (RealEstate, 'real_estates.json'),
                (Address, 'addresses.json'),
                (Municipality, 'municipalities_with_logo.json'),
                (Glossary, 'glossary.json'),
                (ExclusionOfLiability, 'exclusion_of_liability.json')
            ]:
                self._load_sample(class_, file_name)

        finally:
            if self._has_connection():
                self._connection.close()


def _run():
    """
    Loads sample data from the json files with the repository. The method can be called by running
    'load_sample_data' from the virtual environment. Check 'load_sample_data --help' for available options.
    """
    parser = optparse.OptionParser(
        usage='usage: %prog [options]',
        description='Loads sample data into the configured database.'
    )
    parser.add_option(
        '-c', '--configuration',
        dest='configuration',
        metavar='YAML',
        type='string',
        help='The absolute path to the configuration yaml file.'
    )
    parser.add_option(
        '-s', '--section',
        dest='section',
        metavar='SECTION',
        type='string',
        default='pyramid_oereb',
        help='The section which contains configruation (default is: pyramid_oereb).'
    )
    parser.add_option(
        '-d', '--dir',
        dest='directory',
        metavar='DIRECTORY',
        type='string',
        default='sample_data',
        help='The directory containing the sample data (default is: sample_data).'
    )
    parser.add_option(
        '--sql-file',
        type='string',
        help='Generate an SQL file.'
    )
    parser.add_option(
        '--c2ctemplate-style',
        dest='c2ctemplate_style',
        action='store_true',
        default=False,
        help='Is the yaml file using a c2ctemplate style (starting with vars)'
    )
    options, args = parser.parse_args()
    if not options.configuration:
        parser.error('No configuration file set.')
    if options.sql_file is None:
        SampleData(
            options.configuration,
            section=options.section,
            c2ctemplate_style=options.c2ctemplate_style,
            directory=options.directory
        ).load()
    else:
        with codecs.open(options.sql_file, mode='w', encoding='utf-8') as sql_file:
            SampleData(
                options.configuration,
                section=options.section,
                c2ctemplate_style=options.c2ctemplate_style,
                directory=options.directory,
                sql_file=sql_file
            ).load()


if __name__ == '__main__':
    _run()
